# MIT License
# Copyright (c) 2019, INRIA
# Copyright (c) 2019, University of Lille
# All rights reserved.
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import os
import random
import pytest
import pyfakefs

from pyJoules.energy_device.rapl_device import RaplDevice
from .fake_api import FakeAPI


SOCKET_0_DIR_NAME = '/sys/class/powercap/intel-rapl/intel-rapl:0'
SOCKET_1_DIR_NAME = '/sys/class/powercap/intel-rapl/intel-rapl:1'

PKG_0_FILE_NAME = SOCKET_0_DIR_NAME + '/energy_uj'
PKG_0_VALUE = 12345
PKG_1_FILE_NAME = SOCKET_1_DIR_NAME + '/energy_uj'
PKG_1_VALUE = 54321

DRAM_0_DIR_NAME = SOCKET_0_DIR_NAME + '/intel-rapl:0:0'
DRAM_1_DIR_NAME = SOCKET_1_DIR_NAME + '/intel-rapl:1:0'

DRAM_0_FILE_NAME = DRAM_0_DIR_NAME + '/energy_uj'
DRAM_0_VALUE = 6789
DRAM_1_FILE_NAME = DRAM_1_DIR_NAME + '/energy_uj'
DRAM_1_VALUE = 9876

CORE_0_DIR_NAME = SOCKET_0_DIR_NAME + '/intel-rapl:0:1'
CORE_1_DIR_NAME = SOCKET_1_DIR_NAME + '/intel-rapl:1:1'

CORE_0_FILE_NAME = CORE_0_DIR_NAME + '/energy_uj'
CORE_1_FILE_NAME = CORE_1_DIR_NAME + '/energy_uj'


class RaplFS(FakeAPI):

    def __init__(self, fs):
        self.fs = fs
        self.domains_current_energy = {}
        self.domains_energy_file = {}

    def add_domain(self, domain_dir_name, domain_name, domain_id):
        self.fs.create_file(domain_dir_name + '/name', contents=domain_name + '\n')
        energy_value = random.random()
        self.fs.create_file(domain_dir_name + '/energy_uj', contents=str(energy_value) + '\n')
        self.domains_energy_file[domain_id] = domain_dir_name + '/energy_uj'
        self.domains_current_energy[domain_id] = energy_value

    def reset_values(self):
        for key in self.domains_energy_file:
            new_val = random.random()
            self.domains_current_energy[key] = new_val
            with open(self.domains_energy_file[key], 'w') as energy_file:
                energy_file.write(str(new_val) + '\n')

    def get_device_type(self):
        return RaplDevice



@pytest.fixture
def empty_fs(fs):
    """
    filesystem describing a machine with one CPU but no RAPL API
    """
    return RaplFS(fs)


@pytest.fixture
def fs_pkg_one_socket(fs):
    """
    filesystem describing a machine with one CPU and RAPL API for package
    """
    rapl_fs = RaplFS(fs)
    rapl_fs.add_domain(SOCKET_0_DIR_NAME, 'package-0', 'package_0')
    rapl_fs.reset_values()
    return rapl_fs


@pytest.fixture
def fs_pkg_dram_one_socket(fs):
    """
    filesystem describing a machine with one CPU and RAPL API for package and dram
    """
    rapl_fs = RaplFS(fs)
    rapl_fs.add_domain(SOCKET_0_DIR_NAME, 'package-0', 'package_0')
    rapl_fs.add_domain(DRAM_0_DIR_NAME, 'dram', 'dram_0')
    rapl_fs.reset_values()
    return rapl_fs


@pytest.fixture
def fs_pkg_psys_one_socket(fs):
    """
    filesystem describing a machine with one CPU and RAPL API for package and psys
    """
    rapl_fs = RaplFS(fs)
    rapl_fs.add_domain(SOCKET_0_DIR_NAME, 'package-0', 'package_0')
    rapl_fs.add_domain('/sys/class/powercap/intel-rapl/intel-rapl:1', 'psys', 'psys')
    rapl_fs.reset_values()
    return rapl_fs


@pytest.fixture
def fs_pkg_dram_core_one_socket(fs):
    """
    filesystem describing a machine with one CPU and RAPL API for package dram and core
    """
    rapl_fs = RaplFS(fs)
    rapl_fs.add_domain(SOCKET_0_DIR_NAME, 'package-0', 'package_0')
    rapl_fs.add_domain(DRAM_0_DIR_NAME, 'dram', 'dram_0')
    rapl_fs.add_domain(CORE_0_DIR_NAME, 'core', 'core_0')
    rapl_fs.reset_values()
    return rapl_fs

@pytest.fixture
def fs_pkg_dram_uncore_one_socket(fs):
    """
    filesystem describing a machine with one CPU and RAPL API for package dram and core
    """
    rapl_fs = RaplFS(fs)
    rapl_fs.add_domain(SOCKET_0_DIR_NAME, 'package-0', 'package_0')
    rapl_fs.add_domain(DRAM_0_DIR_NAME, 'dram', 'dram_0')
    rapl_fs.add_domain(CORE_0_DIR_NAME, 'uncore', 'uncore_0')
    rapl_fs.reset_values()
    return rapl_fs


@pytest.fixture
def fs_pkg_two_socket(fs):
    """
    filesystem describing a machine with two CPU and RAPL API for package
    """
    rapl_fs = RaplFS(fs)
    rapl_fs.add_domain(SOCKET_0_DIR_NAME, 'package-0', 'package_0')
    rapl_fs.add_domain(SOCKET_1_DIR_NAME, 'package-1', 'package_1')
    rapl_fs.reset_values()
    return rapl_fs


@pytest.fixture
def fs_pkg_dram_two_socket(fs):
    """
    filesystem describing a machine with two CPU and RAPL API for package and dram
    """
    rapl_fs = RaplFS(fs)
    rapl_fs.add_domain(SOCKET_0_DIR_NAME, 'package-0', 'package_0')
    rapl_fs.add_domain(SOCKET_1_DIR_NAME, 'package-1', 'package_1')
    rapl_fs.add_domain(DRAM_0_DIR_NAME, 'dram', 'dram_0')
    rapl_fs.add_domain(DRAM_1_DIR_NAME, 'dram', 'dram_1')
    rapl_fs.reset_values()
    return rapl_fs


@pytest.fixture
def fs_pkg_psys_two_socket(fs):
    """
    filesystem describing a machine with two CPU and RAPL API for package
    """
    rapl_fs = RaplFS(fs)
    rapl_fs.add_domain(SOCKET_0_DIR_NAME, 'package-0', 'package_0')
    rapl_fs.add_domain(SOCKET_1_DIR_NAME, 'package-1', 'package_1')

    rapl_fs.add_domain('/sys/class/powercap/intel-rapl/intel-rapl:2/name', 'psys', 'psys')
    rapl_fs.reset_values()
    return fs



# MIT License
# Copyright (c) 2019, INRIA
# Copyright (c) 2019, University of Lille
# All rights reserved.
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


class PyJoulesException(Exception):
    """
    PyJoules exceptions parent class
    """


class NoSuchDomainError(PyJoulesException):
    """
    Exception raised when a user ask to monitor a domain that is not available on the given device
    """

    def __init__(self, domain_name: str):
        """
        :param domain_name: the domain name that is not available on this device
        """
        PyJoulesException.__init__(self)
        #: the domain name that is not available on this device
        self.domain_name = domain_name


class NoSuchEnergyDeviceError(PyJoulesException):
    """
    Exception raised when a EnergyDevice that does not exist on the current machine is created
    """



# MIT License
# Copyright (c) 2019, INRIA
# Copyright (c) 2019, University of Lille
# All rights reserved.
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import random
import pytest
from pynvml import NVMLError
from pyJoules.energy_device.nvidia_device import NvidiaGPUDomain, NvidiaGPUDevice
from .fake_api import FakeAPI
from mock import patch, Mock

class FakeNvidiaAPI(FakeAPI):

    def __init__(self, number_of_device):
        self.number_of_device = number_of_device

        self.domains_current_energy = None

    def reset_values(self):
        self.domains_current_energy = {}
        for device_id in range(self.number_of_device):
            self.domains_current_energy[str(NvidiaGPUDomain(device_id))] = random.random()

    def get_energy_value(self, device_id):
        return self.domains_current_energy[str(NvidiaGPUDomain(device_id))]

    def get_device_type(self):
        return NvidiaGPUDevice

@pytest.fixture
def no_gpu_api():
    patcher = patch('pynvml.nvmlInit', side_effect=NVMLError(0))
    yield patcher.start()
    patcher.stop()


class MockedHandle(Mock):

    def __init__(self, device_id):
        Mock.__init__(self)
        self.device_id = device_id


@pytest.fixture
def one_gpu_api():
    fake_api = FakeNvidiaAPI(1)
    fake_api.reset_values()

    patcher_init = patch('pynvml.nvmlInit')
    patcher_device_count = patch('pynvml.nvmlDeviceGetCount', return_value=1)
    patcher_get_handle = patch('pynvml.nvmlDeviceGetHandleByIndex', side_effect=MockedHandle)
    patcher_get_energy = patch('pynvml.nvmlDeviceGetTotalEnergyConsumption',
                               side_effect=lambda mocked_handle: fake_api.get_energy_value(mocked_handle.device_id))

    patcher_init.start()
    patcher_device_count.start()
    patcher_get_handle.start()
    patcher_get_energy.start()
    yield fake_api
    patcher_init.stop()
    patcher_device_count.stop()
    patcher_get_handle.stop()
    patcher_get_energy.stop()


@pytest.fixture
def two_gpu_api():
    fake_api = FakeNvidiaAPI(2)
    fake_api.reset_values()

    patcher_init = patch('pynvml.nvmlInit')
    patcher_device_count = patch('pynvml.nvmlDeviceGetCount', return_value=2)
    patcher_get_handle = patch('pynvml.nvmlDeviceGetHandleByIndex', side_effect=MockedHandle)
    patcher_get_energy = patch('pynvml.nvmlDeviceGetTotalEnergyConsumption',
                               side_effect=lambda mocked_handle: fake_api.get_energy_value(mocked_handle.device_id))

    patcher_init.start()
    patcher_device_count.start()
    patcher_get_handle.start()
    patcher_get_energy.start()
    yield fake_api
    patcher_init.stop()
    patcher_device_count.stop()
    patcher_get_handle.stop()
    patcher_get_energy.stop()



# MIT License
# Copyright (c) 2019, INRIA
# Copyright (c) 2019, University of Lille
# All rights reserved.
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from pyJoules.energy_meter import EnergySample


class FakeAPI:
    def reset_values(self):
        raise NotImplementedError()

    def get_device_type(self):
        raise NotImplementedError()


class CorrectTrace:
    """
    Generate the correct trace, consistent with given API
    """

    def __init__(self, domains, fake_api_list, timestamps):
        self.domains = domains
        self.fake_api = {}
        for api in fake_api_list:
            self.fake_api[api.get_device_type()] = api
        self.energy_states = []
        self.correct_timestamps = timestamps
        self.duration_trace = self._compute_duration_trace(timestamps)
        self.tag_trace = []

    def _get_new_energy_values(self):
        new_values = {}
        for domain in self.domains:
            fake_api = self.fake_api[domain.get_device_type()]
            
            new_values[str(domain)] = fake_api.domains_current_energy[str(domain)]
        return new_values

    def _compute_duration_trace(self, timestamps):
        duration_trace = []
        current_ts = timestamps[0]
        for next_ts in timestamps[1:]:
            duration_trace.append(next_ts - current_ts)
            current_ts = next_ts
        return duration_trace

    def add_new_sample(self, tag):
        for _, api in self.fake_api.items():
            api.reset_values()
        self.tag_trace.append(tag)
        self.energy_states.append(self._get_new_energy_values())

    def _compute_energy_trace(self):
        trace = []
        current_state = self.energy_states[0]
        for next_state in self.energy_states[1:]:
            sample = {}
            for domain in self.domains:
                sample[str(domain)] = next_state[str(domain)] - current_state[str(domain)]
            current_state = next_state
            trace.append(sample)
        return trace

    def __iter__(self):
        trace = []
        energy_trace = self._compute_energy_trace()
        zipped_list = zip(energy_trace, self.tag_trace, self.duration_trace,
                          self.correct_timestamps)
        for values, tag, duration, timestamp in zipped_list:
            trace.append(EnergySample(timestamp, tag, duration, values))
        return trace.__iter__()



# MIT License
# Copyright (c) 2019, INRIA
# Copyright (c) 2019, University of Lille
# All rights reserved.
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import pytest

from pyJoules.energy_device.nvidia_device import NvidiaGPUDomain, NvidiaGPUDevice


@pytest.fixture(params=[-1, 0, 1])
def integer_value(request):
    """parametrize a test function with negative, null and positive integers
    """
    return request.param


def test_repr_return_nvidia_gpu_underscore_device_id(integer_value):
    domain = NvidiaGPUDomain(integer_value)
    assert str(domain) == 'nvidia_gpu_' + str(integer_value)


def test_get_device_type_return_NvidiaGPUDevice():
    domain = NvidiaGPUDomain(0)
    assert domain.get_device_type() == NvidiaGPUDevice



# MIT License
# Copyright (c) 2019, INRIA
# Copyright (c) 2019, University of Lille
# All rights reserved.
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from functools import reduce
from operator import add

from .. import EnergySample
from . import EnergyHandler


class PrintHandler(EnergyHandler):

    def process(self, sample: EnergySample):
        """
        Print the given sample on the standard output
        """
        begin_string = f'begin timestamp : {sample.timestamp}; tag : {sample.tag}; duration : {sample.duration}'
        energy_strings = [f'; {domain} : {value}' for domain, value in sample.energy.items()]

        print(reduce(add, energy_strings, begin_string))



# MIT License
# Copyright (c) 2019, INRIA
# Copyright (c) 2019, University of Lille
# All rights reserved.
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import pytest

from pyJoules.exception import NoSuchDomainError, NoSuchEnergyDeviceError
from pyJoules.energy_device import EnergyDeviceFactory
from pyJoules.energy_device.rapl_device import RaplPackageDomain, RaplDramDomain, RaplDevice
from pyJoules.energy_device.nvidia_device import NvidiaGPUDevice, NvidiaGPUDomain
from ...utils.fake_nvidia_api import no_gpu_api, one_gpu_api, two_gpu_api
from ...utils.rapl_fs import fs_pkg_one_socket, fs_pkg_dram_one_socket, empty_fs


def test_create_devices_with_one_rapl_package_domain_return_one_correctly_configured_rapl_device(fs_pkg_dram_one_socket):
    domains = [RaplPackageDomain(0)]
    devices = EnergyDeviceFactory.create_devices(domains)

    assert len(devices) == 1
    assert isinstance(devices[0], RaplDevice)
    assert devices[0].get_configured_domains() == domains


def test_create_devices_with_rapl_package_and_dram_domains_return_one_correctly_configured_rapl_device(fs_pkg_dram_one_socket):
    domains = [RaplPackageDomain(0), RaplDramDomain(0)]
    devices = EnergyDeviceFactory.create_devices(domains)

    assert len(devices) == 1
    assert isinstance(devices[0], RaplDevice)
    assert devices[0].get_configured_domains() == domains


def test_create_devices_with_dram_rapl_domain_without_dram_support_raise_NoSuchDomainError(fs_pkg_one_socket):
    with pytest.raises(NoSuchDomainError):
        EnergyDeviceFactory.create_devices([RaplDramDomain(0)])

def test_create_devices_to_monitor_rapl_without_rapl_api_raise_NoSuchEnergyDeviceError(empty_fs):
    with pytest.raises(NoSuchEnergyDeviceError):
        EnergyDeviceFactory.create_devices([RaplDramDomain(0)])
        
def test_create_devices_to_monitor_gpu0_with_one_gpu_api_return_one_correctly_configured_nvidia_gpu_device(one_gpu_api):
    domains = [NvidiaGPUDomain(0)]
    devices = EnergyDeviceFactory.create_devices(domains)

    assert len(devices) == 1
    assert isinstance(devices[0], NvidiaGPUDevice)
    assert devices[0].get_configured_domains() == domains


def test_create_devices_to_monitor_gpu0_and_gpu1_with_two_gpu_api_return_one_correctly_configured_nvidia_gpu_device(two_gpu_api):
    domains = [NvidiaGPUDomain(0), NvidiaGPUDomain(1)]
    devices = EnergyDeviceFactory.create_devices(domains)

    assert len(devices) == 1
    assert isinstance(devices[0], NvidiaGPUDevice)
    assert devices[0].get_configured_domains() == domains


def test_create_devices_to_monitor_gpu0_without_gpu_api_raise_NoSuchEnergyDeviceError(no_gpu_api):
    with pytest.raises(NoSuchEnergyDeviceError):
        EnergyDeviceFactory.create_devices([NvidiaGPUDomain(0)])

def test_create_devices_to_monitor_gpu1_with_only_one_gpu_api_raise_NoSuchDomainError(one_gpu_api):
    with pytest.raises(NoSuchDomainError):
        EnergyDeviceFactory.create_devices([NvidiaGPUDomain(1)])

def test_create_devices_to_monitor_gpu0_and_package_0_machine_with_two_gpu_and_package_dram_api_create_one_nvidia_gpu_device_and_rapl_device_configured_for_package(two_gpu_api, fs_pkg_dram_one_socket):
    devices = EnergyDeviceFactory.create_devices([RaplPackageDomain(0), NvidiaGPUDomain(0)])

    assert len(devices) == 2
    assert isinstance(devices[1], NvidiaGPUDevice)
    assert devices[1].get_configured_domains() == [NvidiaGPUDomain(0)]
    assert isinstance(devices[0], RaplDevice)
    assert devices[0].get_configured_domains() == [RaplPackageDomain(0)]

        
def test_create_devices_with_default_values_on_machine_with_only_rapl_pkg_and_dram_api_create_one_device_configured_for_dram_and_rapl(fs_pkg_dram_one_socket):
    devices = EnergyDeviceFactory.create_devices()

    assert len(devices) == 1
    assert isinstance(devices[0], RaplDevice)
    assert devices[0].get_configured_domains() == [RaplPackageDomain(0), RaplDramDomain(0)]


def test_create_devices_with_default_values_on_machine_with_only_one_gpu_create_one_nvidia_gpu_device(one_gpu_api, empty_fs):
    devices = EnergyDeviceFactory.create_devices()

    assert len(devices) == 1
    assert isinstance(devices[0], NvidiaGPUDevice)
    assert devices[0].get_configured_domains() == [NvidiaGPUDomain(0)]


def test_create_devices_with_default_values_on_machine_with_one_gpu_and_package_api_create_one_nvidia_gpu_device_and_rapl_device_configured_for_package(one_gpu_api, fs_pkg_one_socket):
    devices = EnergyDeviceFactory.create_devices()

    assert len(devices) == 2
    assert isinstance(devices[1], NvidiaGPUDevice)
    assert devices[1].get_configured_domains() == [NvidiaGPUDomain(0)]
    assert isinstance(devices[0], RaplDevice)
    assert devices[0].get_configured_domains() == [RaplPackageDomain(0)]
